import 'package:feedify/feedsdk.dart';
import 'package:feedify/src/model/ad/model_ad_request.dart';
import 'package:feedify/src/model/model_device_app.dart';
import 'package:feedify/src/model/notification/model_notification.dart';
import 'package:feedify/src/model/register/register_request.dart';
import 'package:feedify/src/service/repository.dart';
import 'package:feedify/src/storage/app-prefs.dart';
import 'package:firebase_messaging/firebase_messaging.dart';

import '../../logs.dart';

class FeedApiManager {
  static final FeedApiManager _singleton = FeedApiManager._internal();

  FeedApiManager._internal();

  final _repository = Repository();

  _FeedApiManager() {}

  static FeedApiManager getInstance() {
    return _singleton;
  }

  void register(AppPreferences? appPreferences) async {
    String? fcmKey = await appPreferences?.getString(AppPreferences.KEY_FCM);
    if (fcmKey != null && fcmKey.isNotEmpty) {
      _register(fcmKey);
    }


  }

  void _register(String fcmKey) async {
    var mobileDevice = await ModelDeviceApp.getInstance();
    try {
      if (mobileDevice != null) {
        _repository.register(_getRegisterRequest(fcmKey, mobileDevice));
      }
    } catch (e) {
      Logs.d(e.toString());
    }
  }

  RegisterRequest _getRegisterRequest(
      String fcmKey, ModelDeviceApp mobileDevice) {
    return RegisterRequest(fcmKey, FeedSDK.feedUser, FeedSDK.feedKey,
        FeedSDK.feedDomain, mobileDevice.getUUID());
  }

  Future<ModelNotification?> fetchAd(
      Map<String, dynamic> notificationData) async {


    //Fluttertoast.showToast(msg: "msg222");

    var requestModel = await _getModelAdRequest(notificationData);

    //Fluttertoast.showToast(msg: "msg223");

    return await _repository.fetchAd(requestModel);
  }

  Future<ModelNotification?> fetchEmptyDataNotification(
      Map<String, dynamic> notificationData) async {


    //Fluttertoast.showToast(msg: "msg222");

    var requestModel = await _getModelAdRequest(notificationData);

    //Fluttertoast.showToast(msg: "msg223");

     var token2 = await getToken();

    return await _repository.fetchEmptyDataNotification(token2 ?? "");
  }


  Future<ModelAdRequest> _getModelAdRequest(
      Map<String, dynamic> notificationData) async {

    var token = "" ;
    try{

       token = await FirebaseMessaging.instance.getToken() ?? "";

    }catch(e){

      token = await   getToken() ?? "";
    }

    return ModelAdRequest(
        notificationData, 'https://fcm.googleapis.com/fcm/send/$token');
  }

  Future<String?> getToken() async {
    return await AppPreferences().getString(AppPreferences.KEY_FCM);
  }

  void deliveryCount(String id, String type) {
    try {
      _repository.countDelivery(id, type);
    } catch (e) {
      Logs.d(e.toString());
    }
  }

  void callClickApi(String type, String id) {
    try {
      _repository.callClickApi(id, type);
    } catch (e) {
      Logs.d(e.toString());
    }
  }
}
